#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "dstring.h"

#define ENDSTR '\0'

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : DSTRING (CONSTRUCTOR)                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Initialisierung des dynamischen Strings        *
// *                (ohne Datenzuweisung)                          *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : keiner                                         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
dstring::dstring ()
{
	//---------------------------------------------------------------
	// Alle Pointer sollten stets einen gltigen Wert haben. Wird
	// ptr nicht auf NULL gesetzt, sind schwere Abstrze die Folge, 
	// da ptr dann einen Zufallswert enthlt, der als freizugebender 
	// Speicherblock interpretiert wird, obwohl er nicht dynamisch
	// mit dem Betriebssystem vereinbart wurde. Dadurch wrden noch
	// verwendete Programmteile freigegeben, beim nchsten Zugriff
	// einen dieser Teile bricht das Betriebssystem ab, da ein 
	// illegaler Speicherzugriff erfolgt
	//---------------------------------------------------------------
	ptr = NULL;
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : DSTRING (CONSTRUCTOR)                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Initialisierung des dynamischen Strings        *
// *                (mit Datenzuweisung ber String)               *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : toset - Initialisierungsstring                 *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : keiner                                         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
dstring::dstring (char * toset)
{
	//---------------------------------------------------------------
	// Alle Pointer sollten stets einen gltigen Wert haben. (siehe
	// Standard-Constructor)
	//---------------------------------------------------------------
	ptr = NULL;

	//---------------------------------------------------------------
	// Da es sich beim Parameter um einen Stringpointer handelt, 
	// knnen wir einfach die dafr vorgesehene SET-Methode aufrufen
	//---------------------------------------------------------------
	set (toset);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : DSTRING (CONSTRUCTOR)                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Initialisierung des dynamischen Strings        *
// *                (mit Datenzuweisung ber DSTRING-Objekt)       *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : toset - zur Initialisierung zu kopierendes     *
// *                        DSTRING-Objekt                         *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : keiner                                         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
dstring::dstring (dstring & toset)
{
	//---------------------------------------------------------------
	// Alle Pointer sollten stets einen gltigen Wert haben. (siehe
	// Standard-Constructor)
	//---------------------------------------------------------------
	ptr = NULL;

	//---------------------------------------------------------------
	// Da es sich beim Parameter um ein DSTRING-Objekt handelt, ist 
	// lediglich der Stringpointer zu ermitteln und dann kann man 
	// einfach die dafr vorgesehene SET-Methode aufrufen
	// Der Zugriff auf die PRIVATE Membervariable TOSET.PTR ist 
	// erlaubt, da wir uns hier innerhalb der DSTRING-Klasse befinden
	//---------------------------------------------------------------
	set (toset.ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : ~DSTRING (DESTRUCTOR)                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Freigabe des dynamisch belegten Speichers      *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : keiner                                         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
dstring::~dstring ()
{
	//---------------------------------------------------------------
	// Freigabe (DELETE) ist nur dann sinnvoll, wenn auch was darin
	// steht !
	// VORSICHT, wenn NULL-Pointer freigegeben wird besteht bei
	// einigen, sehr wenigen Compilern Absturzgefahr (laut ANSI-
	// Definition ist die Freigabe von NULL erlaubt und unkritisch)
	// Bitte beachten, da  hier DELETE [] benutzt wird, da auch 
	// NEW [] verwendet wurde. Die meisten Compiler verarbeiten zwar
	// auch ein einfaches DELETE korrekt, laut ANSI ist das Verhalten
	// des Programms (bei Allokation mit der Arrayvariante NEW [] und
	// Deallokation mit der einfachen Variante DELETE) jedoch nicht
	// definiert
	//---------------------------------------------------------------
	if (ptr) delete [] ptr;
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : LOWER                                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Inhalt des DSTRING-Objekts in Kleinschrift     *
// *                umwandeln                                      *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::lower (void)
{
	char *help = ptr;     // Hilfszeiger

	//---------------------------------------------------------------
	// Wenn keine Daten vorhanden sind, kann man auch nichts tun
	//---------------------------------------------------------------
	if (!ptr) return ("");

	//---------------------------------------------------------------
	// Wenn Daten vorhanden sind, dann umsetzen (insbesondere die
	// deutschen Sonderzeichen
	// Die FOR-Schleife bricht ab, wenn HELP auf das abschlie ende
	// STRINGENDE (ASCII-NULL) zeigt (*HELP ist FALSE)
	//---------------------------------------------------------------
	for (; *help; help++)
	{
		//------------------------------------------------------------
		// Wenn der Inhalt auf den der Hilfspointer HELP zeigt, im 
		// ASCII-Bereich 65..90 (A..Z) liegt, dann 32 aufaddieren
		// (ASCII-Bereich fr a..z)
		// Die deutschen Umlaute liegen naturgem sonstwo, daher ist
		// ein SWITCH das Mittel der Wahl
		// ACHTUNG: die ASCII-Werte liegen bei DOS und Windows an
		// unterschiedlichen Positionen, daher fr die Umlaute NIEMALS
		// ASCII-Werte direkt angeben !!!!
		//------------------------------------------------------------
		if ((*help > 64) && (*help < 91)) *help += 32;
		else switch (*help)
		{
			case '' : *help = ''; break;
			case '' : *help = ''; break;
			case '' : *help = ''; break;
		}
	}
	//---------------------------------------------------------------
	// Zum Schlu PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : UPPER                                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Inhalt des DSTRING-Objekts in Gro schrift      *
// *                umwandeln                                      *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::upper (void)
{
	char *help = ptr;     // Hilfszeiger

	//---------------------------------------------------------------
	// Wenn keine Daten vorhanden sind, kann man auch nichts tun
	//---------------------------------------------------------------
	if (!ptr) return ("");

	//---------------------------------------------------------------
	// Wenn Daten vorhanden sind, dann umsetzen (insbesondere die
	// deutschen Sonderzeichen
	// Die FOR-Schleife bricht ab, wenn HELP auf das abschlie ende
	// STRINGENDE (ASCII-NULL) zeigt (*HELP ist FALSE)
	//---------------------------------------------------------------
	for (; *help; help++)
	{
		//------------------------------------------------------------
		// Wenn der Inhalt auf den der Hilfspointer HELP zeigt, im 
		// ASCII-Bereich 97..122 (a..z) liegt, dann 32 subtrahieren
		// (ASCII-Bereich fr A..Z)
		// Die deutschen Umlaute liegen naturgem sonstwo, daher ist
		// ein SWITCH das Mittel der Wahl
		// ACHTUNG: die ASCII-Werte liegen bei DOS und Windows an
		// unterschiedlichen Positionen, daher fr die Umlaute NIEMALS
		// ASCII-Werte direkt angeben !!!!
		//------------------------------------------------------------
		if ((*help > 96) && (*help < 123)) *help -= 32;
		else switch (*help)
		{
			case '' : *help = ''; break;
			case '' : *help = ''; break;
			case '' : *help = ''; break;
		}
	}
	//---------------------------------------------------------------
	// Zum Schlu PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : LTRIM                                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Leerzeichen (Blanks) am Stringanfang entfernen *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::ltrim (void)
{
	char *help = ptr;     // Hilfszeiger
	dstring temp;         // tempor rer DSTRING zum umkopieren

	//---------------------------------------------------------------
	// Wenn keine Daten vorhanden sind (PTR ist NULL), kann man auch
	// nichts tun. 
	//---------------------------------------------------------------
	if (!ptr) return ("");

	//---------------------------------------------------------------
	// Wenn der String zwar gesetzt (PTR ist ungleich NULL), aber 
	// leer ist (Stringlnge Null) braucht man auch nichts tun, da es
	// keine Leerzeichen geben kann, die abzuschneiden sind 
	//---------------------------------------------------------------
	if (len (ptr) < 1) return (ptr);

	//---------------------------------------------------------------
	// Wenn der ASCII-Wert des aktuellen Zeichens kleiner ist als 33,
	// dann ist es ein Whitespace-Zeichen und soll abgeschnitten
	// werden. 
	// Letztlich wird hier nach dem ersten nicht-Whitespace gesucht.
	// Die Schleife wird beendet, wenn *HELP auf das Stringende
	// (ASCII-Null) zeigt (*HELP ist FALSE)
	//---------------------------------------------------------------
   while ((*help < 33) && (*help)) help++;

	//---------------------------------------------------------------
	// Wenn HELP jetzt auf dem Stringende-Zeichen steht, dann enthlt
	// die Zeichenkette nur Whitespace-Zeichen und mu zurckgesetzt
	// werden. Ist dies nicht der Fall, darf der DTSTRING nur die 
	// Zeichen ab der Position HELP enthalten
	// Um dies zu erreichen, mu man einen neuen Speicherbereich an-
	// fordern (mit der Lnge des Reststrings ab Position HELP), ab
	// der Position HELP in diesen neuen Speicherbereich hinein-
	// kopieren und den alten Speicherbereich freigeben. Ziemlich
	// mhselig, aber praktischerweise knnen wird dazu einfach die
	// SET-Methode eines TEMPORREN DSTRING-Objekts verwenden
	// Selbst dieses knnen wir uns sparen, wenn es keine Daten zu 
	// bernehmen gibt...
	//---------------------------------------------------------------
	if (ENDSTR == *help) temp.set ("");
	else temp.set (help);

	set (temp.ptr);

	//---------------------------------------------------------------
	// Zum Schlu PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : RTRIM                                          *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Leerzeichen (Blanks) am Stringende entfernen   *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::rtrim (void)
{
	char *help = ptr;     // Hilfszeiger
	dstring temp;         // tempor rer DSTRING zum umkopieren
	int length;           // L nge des zu bearbeitenden Strings
	int i;                // Z hlvariable

	//---------------------------------------------------------------
	// Wenn keine Daten vorhanden sind (PTR ist NULL), kann man auch
	// nichts tun. 
	//---------------------------------------------------------------
	if (!ptr) return ("");

	//---------------------------------------------------------------
	// Wenn der String zwar gesetzt (PTR ist ungleich NULL), aber 
	// leer ist (Stringl nge Null) braucht man auch nichts tun, da es
	// keine Leerzeichen geben kann, die abzuschneiden sind 
	//---------------------------------------------------------------
	i = length = len (ptr);
	if (i < 1) return (ptr);

	//---------------------------------------------------------------
	// Jetzt wird der String von hinten nach vorn durchsucht.
	// dabei merken wir uns die Position des ersten NICHT-Leer-
	// zeichens, das wir finden.
	// Es ist allerdings darauf zu achten, da  man (bei Strings die
	// NUR Leerzeichen enthalten) nicht vergit bei Index Null anzu-
	// halten...
	//---------------------------------------------------------------
	while ((help [i] < 33) && (i >= 0)) i--;

	//---------------------------------------------------------------
	// Wenn HELP jetzt auf dem Index Null steht, dann enthlt die
	// Zeichenkette nur Whitespace-Zeichen und mu zurckgesetzt
	// werden. Ist dies nicht der Fall, darf der DTSTRING nur die 
	// Zeichen bis zur Position HELP enthalten
	// Da Help entweder auf dem ersten Nicht-Leerzeichen steht, oder
	// auf dem Index -1 (gem Postdekrement in der Schleife), kann
	// man einfach auf Position i+1 ein Stringendesymbol schreiben.
	// Dies funktioniert auch, wenn PTR gar keine abzuschneidenden
	// Leerzeichen enthlt, da die Schleife bei dem Index beginnt, 
	// der der Lnge des Strings entspricht (dies ist IMMER das
	// Stringendezeichen). Es wird dann also nur das Stringende-
	// symbol berschrieben.
	//---------------------------------------------------------------
	help[i+1] = 0;
   
	//---------------------------------------------------------------
	// Um dies zu erreichen, mu man einen neuen Speicherbereich an-
	// fordern (mit der Lnge des Reststrings ab Position HELP), ab
	// der Position HELP in diesen neuen Speicherbereich hinein-
	// kopieren und den alten Speicherbereich freigeben. Ziemlich
	// mhselig, aber praktischerweise knnen wird dazu einfach die
	// SET-Methode eines TEMPORREN DSTRING-Objekts verwenden
	// Der Vergleich von LENGTH und I+1 erspart das kopieren unver-
	// nderter Zeichenketten
	//---------------------------------------------------------------
	if (length != i+1)
	{
		temp.set (ptr);
		set (temp.ptr);
	}
	
	//---------------------------------------------------------------
	// Zum Schlu PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : ALLTRIM                                        *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Leerzeichen (Blanks) am Stringende und am      *
// *                Stringanfang entfernen                         *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : keine                                          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::alltrim (void)
{
	//---------------------------------------------------------------
	// Wenn keine Daten vorhanden sind (PTR ist NULL), kann man auch
	// nichts tun. 
	//---------------------------------------------------------------
	if (!ptr) return ("");

	//---------------------------------------------------------------
	// ALLTRIM ist nichts anderes als eine Kombination von LTRIM und
	// RTRIM, also knnen wir die entsprechenden Methoden auch nutzen
	//---------------------------------------------------------------
	rtrim ();
	ltrim ();

	//---------------------------------------------------------------
	// Zum Schlu  PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : SET                                            *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Setzen des dynamischen Strings                 *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : toset - auf PTR zu kopierender Text            *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::set (char *toset)
{
	//---------------------------------------------------------------
	// Dies ist eines der beiden "Herzstcke" der Klasse (das andere
	// ist die Methode CONCAT), hier geschieht die eigentliche Allo-
	// kation und Deallokation des dynamischen Speichers. Auch ist
	// hier aus Effizienzgrnnden eine Menge Pointerarbeit ntig
	//---------------------------------------------------------------
	long size = len (toset) + 1;  // Stringlnge = Speicherbedarf !!
	char *help = NULL;            // Hilfszeiger fr Kopierarbeit

	//---------------------------------------------------------------
	// Wenn der DSTRING schon gesetzt war, enthlt er Daten, die zu
	// lschen sind. Auch mu der zugehrige Speicher freigegeben
	// werden. Die Deallokation sollte vor der Neu-Allokation 
	// erfolgen, da es bei hoher Speicherbelastung sonst vorkommen
	// kann, da  nicht gengend Speicher vorhanden ist um beide 
	// Speicherbereiche gleichzeitig zu belegen
	//---------------------------------------------------------------
	if (ptr) delete [] ptr;

	//---------------------------------------------------------------
	// Jetzt neuen Speicherplatz der bentigten Gre anfordern
	//---------------------------------------------------------------
	ptr = new char [size];

	//---------------------------------------------------------------
	// Ist nicht gengend Speicher vorhanden, erhalten wir einen
	// NULL-Pointer zurck
	//---------------------------------------------------------------
	if (!ptr) return ("");

	//---------------------------------------------------------------
	// Ist genug Speicher vorhanden, dann aus dem bergebenen Pointer
	// in den Speicherbereich hinberkopieren. Da wir PTR nicht ver-
	// ndern drfen, benutzen wir einen Hilfszeiger (HELP) auf dem
	// neugeschaffenen Speicherbereich. TOSET drfen wir verndern,
	// da dieser Pointer CALL BY VALUE bergeben wurde und TOSET an-
	// schlieend in dieser Methode nicht mehr bentigt wird
	//---------------------------------------------------------------
	for (help=ptr; *toset; help++, toset++) *help = *toset;

	//---------------------------------------------------------------
	// nicht vergessen das Stringende zu setzen...
	//---------------------------------------------------------------
	ptr [size-1] = 0;
	
	//---------------------------------------------------------------
	// Zum Schlu PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : CONCAT                                         *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Text an einen dynamischen String anh ngen      *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : toadd - an PTR anzuh ngender Text              *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::concat (char *toadd)
{
	//---------------------------------------------------------------
	// Dies ist eines der beiden "Herzstcke" der Klasse (das andere
	// ist die Methode SET), hier geschieht die eigentliche Allo-
	// kation und Deallokation des dynamischen Speichers. Auch ist
	// hier aus Effizienzgrnden eine Menge Pointerarbeit ntig
	//---------------------------------------------------------------
	long  size;                   // Stringlnge = Speicherbedarf !!
	char *nptr  = NULL;           // Zeiger auf neuen Speicherbereich
	char *nhelp = NULL;           // Hilfszeiger fr Kopierarbeit
	char *ohelp = NULL;           // Hilfszeiger fr Kopierarbeit

	//---------------------------------------------------------------
	// Wenn PTR auf NULL zeigt, dann ist das Anhngen eigentlich nur
	// ein Setzen (SET)...
	//---------------------------------------------------------------
	if (!ptr)
	{
		set (toadd);
		return (ptr);
	}

	//---------------------------------------------------------------
	// Zuerst mu der bentigte Speicherplatz der kombinierten 
	// Zeichenketten ermittelt werden (plus abschlieende ASCII-Null)
	//---------------------------------------------------------------
	size = len (ptr) + len (toadd) + 1;

	//---------------------------------------------------------------
	// Anders als bei SET knnen wir hier den Speicher von PTR nicht
	// zuerst freigeben, da der Inhalt von PTR noch bentigt wird.
	// Also zuerst den Speicher besorgen und auf einen Hilfszeiger
	// zuweisen.
	//---------------------------------------------------------------
	nptr = new char [size];

	//---------------------------------------------------------------
	// WAS TUN ? - Wenn nicht mehr gengend Speicher vorhanden ist
	// Mglich wre eine Rckgabe von NULL, das Setzen einer Fehler-
	// variable oder den String unverndert zu lassen. Dies ist eine
	// Frage der eigenen Vorlieben. Hier lassen wir den String ein-
	// fach unverndert...
	//---------------------------------------------------------------
	if (!nptr) return ("");

	//---------------------------------------------------------------
	// Wenn wir den erforderlichen Speicher erhalten haben, dann
	// knnen wir jetzt beginnen die Inhalte zu bertragen. Zunchst
	// der Teil, der bisher schon in PTR stand. Dazu verwenden wir
	// zwei Hilfszeiger, weil weder PTR noch NPTR verndert werden
	// drfen.
	//---------------------------------------------------------------
	for (nhelp=nptr, ohelp=ptr; *(ohelp); nhelp++, ohelp++) 
	   *nhelp=*ohelp;
   
	//---------------------------------------------------------------
	// Anschlieend der Teil wird der als Parameter bergebene String
	// hinter die PTR-Kopie in NPTR geschrieben. Dazu verwenden 
	// wir nur noch NHELP als Hilfszeiger (der steht ja hinter dem
	// zuletzt kopierten Zeichen. TOADD dnrfen wir verndern,
	// da dieser Pointer CALL BY VALUE bergeben wurde und TOADD an-
	// schlieend in dieser Methode nicht mehr bentigt wird
	//---------------------------------------------------------------
	for (; *(toadd); nhelp++, toadd++) *nhelp = *toadd;
	*nhelp = 0;

	//---------------------------------------------------------------
	// Jetzt wird der alte Inhalt von PTR nicht mehr gebraucht und
	// kann freigegeben werden...
	//---------------------------------------------------------------
	delete [] ptr;
   
	//---------------------------------------------------------------
	// Schnell noch den Pointer von NPTR auf PTR sichern und schon
	// ist es geschafft...
	//---------------------------------------------------------------
	ptr = nptr;
	
	//---------------------------------------------------------------
	// Zum Schlu PTR zurckgeben, damit der String z.B. direkt an
	// cout weitergegeben werden kann
	//---------------------------------------------------------------
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : LEN                                            *
// * SICHTBARKEIT : private                                        *
// * BESCHREIBUNG : L nge eines Strings ermitteln                  *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : tocount - Text, dessen L nge zu ermitteln ist  *
// * ABH NGIGKEIT : keine                                          *
// * RETURN-WERT  : long    - Lnge des Texts                      *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
long dstring::len (char *tocount) const
{
	char *help = tocount;

	//---------------------------------------------------------------
	// Startadresse minus Endadresse ist natrlich die Gre in Bytes
	// (ohne String-Ende-Symbol)
	//---------------------------------------------------------------
	for (; *help; help++);
	return (help-tocount);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR =                                     *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Zuweisung zweier DSTRING-Objekte berladen     *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : tocopy - zu kopierendes DSTRING-Objekt         *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::operator= (dstring & tocopy)
{
	//---------------------------------------------------------------
	// Die Zuweisung eines DSTRING-Objektes soll dem Setzen des 
	// DSTRING-Objektes entsprechen.
	// Die Zuweisung wird eigentlich automatisch vom C-Compiler er-  
	// zeugt. Da TOCOPY jedoch ein Pointer auf einen Speicherbereich
	// ist, und man nicht die Kopie des Pointers mchte, sondern 
	// eine Kopie des Inhaltes, mu man diesen Kopiervorgang von Hand
	// durchfnhren.
	//---------------------------------------------------------------
	set (tocopy.ptr);
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR =                                     *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Zuweisung eines Strings berladen              *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : tocopy - zu kopierender String                 *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::operator= (char *tocopy)
{
	//---------------------------------------------------------------
	// Die Zuweisung eines Strings soll dem Setzen des DSTRING-
	// Objektes entsprechen.
	// Die Zuweisung wird eigentlich automatisch vom C-Compiler er-  
	// zeugt. Da TOCOPY jedoch ein Pointer auf einen Speicherbereich
	// ist, und man nicht die Kopie des Pointers mchte, sondern 
	// eine Kopie des Inhaltes, mu  man diesen Kopiervorgang von Hand
	// durchfhren.
	//---------------------------------------------------------------
	set (tocopy);
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR +                                     *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Anh ngen eines DSTRING-Objektes                *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : toadd  - anzuh ngendes DSTRING-Objekt          *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::operator+ (dstring & toadd)
{
	//---------------------------------------------------------------
	// Der Operator + soll die gleiche Wirkung haben wie die Methode
	// CONCAT
	//---------------------------------------------------------------
	concat (toadd.ptr);
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR +                                     *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Anh ngen eines Strings                         *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : toadd  - anzuh ngender String                  *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char * - Adresse von des Members PTR           *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char *dstring::operator+ (char *toadd)
{
	//---------------------------------------------------------------
	// Der Operator + soll die gleiche Wirkung haben wie die Methode
	// CONCAT
	//---------------------------------------------------------------
	concat (toadd);
	return (ptr);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR ==                                    *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Vergleich mit einem String                     *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : tocheck - zu vergleichender String             *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : int - TRUE wenn identisch, sonst FALSE         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
int dstring::operator== (char *tocheck)
{
	//---------------------------------------------------------------
	// Der Vergleich wird eigentlich automatisch vom C-Compiler er-  
	// zeugt. Da TOCHECK jedoch ein Pointer auf einen Speicherbereich
	// ist, und man nicht die Pointer miteinander vergleichen mchte,
	// sondern ihre Inhalte, mu  man den Vergleich von Hand durch-
	// fhren.
	//---------------------------------------------------------------
	int nlen = len (ptr);

	//---------------------------------------------------------------
	// Wenn die beiden Zeichenketten nicht gleich lang sind, knnen
	// sie nicht identisch sein...
	//---------------------------------------------------------------
	if (nlen != len (tocheck)) return (0);

	//---------------------------------------------------------------
	// Sind sie gleich lang, so vergleichen wir Zeichen fr Zeichen,
	// bis wir einen Unterschied gefunden haben oder der String zu-
	// ende ist
	//---------------------------------------------------------------
	while (nlen >= 0)
	{
		if (tocheck[nlen] != ptr[nlen]) return (0);
		nlen--;
	}
	
	return (1);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR <=                                    *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Vergleich mit einem String                     *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : tocheck - zu vergleichender String             *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : int - TRUE wenn identisch, sonst FALSE         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
int dstring::operator<= (char *tocheck)
{
	//---------------------------------------------------------------
	// Dieser Vergleich wird NICHT automatisch vom C-Compiler er-
	// zeugt, man mu den Vergleich immer von Hand durchfhren.
	//---------------------------------------------------------------
	int nlen = len (ptr);

	//---------------------------------------------------------------
	// Wir vergleichen Zeichen fr Zeichen, bis wir einen Unterschied
	// gefunden haben oder der String zuende ist
	//---------------------------------------------------------------
	while (nlen >= 0)
	{
		if (ptr[nlen] > tocheck[nlen]) return (0);
		nlen--;
	}
	return (1);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR >=                                    *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Vergleich mit einem String                     *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 27.03.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : tocheck - zu vergleichender String             *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : int - TRUE wenn identisch, sonst FALSE         *
// *---------------------------------------------------------------*
// * 27.03.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
int dstring::operator>= (char *tocheck)
{
	//---------------------------------------------------------------
	// Dieser Vergleich wird NICHT automatisch vom C-Compiler er-
	// zeugt, man mu den Vergleich immer von Hand durchfhren.
	//---------------------------------------------------------------
	int nlen = len (ptr);

	//---------------------------------------------------------------
	// Wir vergleichen Zeichen fr Zeichen, bis wir einen Unterschied
	// gefunden haben oder der String zuende ist
	//---------------------------------------------------------------
	while (nlen >= 0)
	{
		if (ptr[nlen] < tocheck[nlen]) return (0);
		nlen--;
	}

	return (1);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR []                                    *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Zugriff auf einen Character                    *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 08.11.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : idx - index des zu holenden Zeichens           *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char& - Referenz auf das Zeichen im String,    *
// *                so da dieses gelesen und geschrieben werden   *
// *                kann                                           *
// *---------------------------------------------------------------*
// * 08.11.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char& dstring::operator[] (int idx)
{
	int nlen = len (ptr);

	//---------------------------------------------------------------
	// Wenn ber Indexklammern auf einen einzelnen Buchstaben zuge-
	// griffen wird, dann ist sicherzustellen, da der Bereich des
	// Stringsnicht verlassen wird. Ist der Index < 0, so wird ange-
	// nommen, da es sich um index 0 handeln soll. Ist der Index 
	// jenseits des letzten Zeichens, so wird das letzte Zeichen 
	// zurckgegeben
	//---------------------------------------------------------------
	if (idx < 0) return (ptr[0]);
	if (idx > nlen-1) return (ptr[nlen-1]);
	return (ptr[idx]);
}

// *****************************************************************
// * KLASSE       : DSTRING                                        *
// * METHODE      : OPERATOR +                                     *
// * SICHTBARKEIT : public                                         *
// * BESCHREIBUNG : Zugriff auf einen Reststring                   *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 08.11.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : idx - index auf Beginn des Reststrings         *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : char* - Zeiger auf den Reststring ab Zeichen   *
// *                idx                                            *
// *---------------------------------------------------------------*
// * 08.11.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
char* dstring::operator+ (int idx)
{
   int nlen = len (ptr);

   if (idx < 0) return (ptr);
   if (idx > nlen-1) return (ptr+nlen);
   return (ptr+idx);
}

// *****************************************************************
// * KLASSE       : -------                                        *
// * METHODE      : OPERATOR <<                                    *
// * SICHTBARKEIT : ------                                         *
// * BESCHREIBUNG : Streamausgabe einer dstring-Zeichenkette       *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 08.11.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : ostream& - implizite bergabe des streams      *
// *                z        - dstring-Zeichenkette im stream      *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : ostream& - stream, zwecks Aufreihung           *
// *---------------------------------------------------------------*
// * 08.11.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
ostream& operator<< (ostream& s, dstring z)
{
   s << z.ptr;
   return (s);
}

// *****************************************************************
// * KLASSE       : -------                                        *
// * METHODE      : OPERATOR >>                                    *
// * SICHTBARKEIT : ------                                         *
// * BESCHREIBUNG : Streameingabe einer dstring-Zeichenkette       *
// *---------------------------------------------------------------*
// * VERSION      : 1.0                                            *
// * ERSTELLUNG   : 08.11.1997                                     *
// * ERSTELLER    : H. Gorski                                      *
// * LETZTE  NDER.:                                                *
// *  NDERER      :                                                *
// *---------------------------------------------------------------*
// * GLOBALE VAR. : keine                                          *
// * PARAMETER    : istream& - implizite bergabe des streams      *
// *                z        - dstring-Zeichenkette im stream      *
// * ABH NGIGKEIT : keine                                          *
// * RETURNWERT   : istream& - stream, zwecks Aufreihung           *
// *---------------------------------------------------------------*
// * 08.11.1997   : H. Gorski                                      *
// *                Erstellung                                     *
// *****************************************************************
istream& operator>> (istream& s, dstring &z)
{
	char x [257] = "";

	s.getline (x, 257);
	z.set (x);

	while (strlen (x) == 256)
	{
		s.getline (x, 257);
		z.concat (x);
	}
   return (s);
}

float dstring::GetFloat (int Offset)
{
	long l = length();
	if (ptr)
   {
   	if (l <= Offset) Offset = l;
      return ((float)(atof (ptr+Offset)));
   }
}

double dstring::GetDouble (int Offset)
{
	long l = length();
	if (ptr)
   {
   	if (l <= Offset) Offset = l;
      return (atof (ptr+Offset));
   }
}

long double dstring::GetLongDouble (int Offset)
{
	long l = length();
	if (ptr)
   {
   	if (l <= Offset) Offset = l;
      return (_atold (ptr+Offset));
   }
}

int dstring::GetInt (int Offset)
{
	long l = length();
	if (ptr)
   {
   	if (l <= Offset) Offset = l;
      return (atoi (ptr+Offset));
   }
}


